"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.api = void 0;
const chai_1 = require("chai");
const client_1 = require("@deep-foundation/hasura/client");
const client_2 = require("../imports/client");
const handlers_1 = require("../imports/handlers");
const api_1 = require("@deep-foundation/hasura/api");
const sql_1 = require("@deep-foundation/hasura/sql");
const _1655979260869_sync_handlers_1 = require("../migrations/1655979260869-sync-handlers");
const debug_1 = __importDefault(require("debug"));
const debug = (0, debug_1.default)('deeplinks:tests:sync-handlers');
const log = debug.extend('log');
const error = debug.extend('error');
const namespaces = debug_1.default.disable();
debug_1.default.enable(`${namespaces ? `${namespaces},` : ``}${error.namespace}`);
const delay = time => new Promise(res => setTimeout(res, time));
const HASURA_PATH = 'localhost:8080';
const HASURA_SSL = 0;
const HASURA_SECRET = 'myadminsecretkey';
exports.api = new api_1.HasuraApi({
    path: HASURA_PATH,
    ssl: !!+HASURA_SSL,
    secret: HASURA_SECRET,
});
const apolloClient = (0, client_1.generateApolloClient)({
    path: `${process.env.DEEPLINKS_HASURA_PATH}/v1/graphql`,
    ssl: !!+process.env.DEEPLINKS_HASURA_SSL,
    secret: process.env.DEEPLINKS_HASURA_SECRET,
});
const deep = new client_2.DeepClient({ apolloClient });
beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
    console.log('manual remigrating...');
    yield exports.api.sql(`${_1655979260869_sync_handlers_1.createPrepareFunction}`);
    yield exports.api.sql(`${_1655979260869_sync_handlers_1.createDeepClientFunction}`);
    yield exports.api.sql(_1655979260869_sync_handlers_1.dropSyncInsertTrigger);
    yield exports.api.sql(_1655979260869_sync_handlers_1.dropSyncInsertTriggerFunction);
    yield exports.api.sql(_1655979260869_sync_handlers_1.dropSyncUpdateTrigger);
    yield exports.api.sql(_1655979260869_sync_handlers_1.dropSyncUpdateTriggerFunction);
    yield exports.api.sql(_1655979260869_sync_handlers_1.dropSyncDeleteTrigger);
    yield exports.api.sql(_1655979260869_sync_handlers_1.dropSyncDeleteTriggerFunction);
    yield exports.api.sql(_1655979260869_sync_handlers_1.createSyncInsertTriggerFunction);
    yield exports.api.sql(_1655979260869_sync_handlers_1.createSyncInsertTrigger);
    yield exports.api.sql(_1655979260869_sync_handlers_1.createSyncUpdateTriggerFunction);
    yield exports.api.sql(_1655979260869_sync_handlers_1.createSyncUpdateTrigger);
    yield exports.api.sql(_1655979260869_sync_handlers_1.createSyncDeleteTriggerFunction);
    yield exports.api.sql(_1655979260869_sync_handlers_1.createSyncDeleteTrigger);
}));
describe('sync handlers', () => {
    describe('Prepare fuction', () => {
        it(`handleInsert`, () => __awaiter(void 0, void 0, void 0, function* () {
            var _a, _b, _c;
            const handlerId = yield deep.id('@deep-foundation/core', 'HandleInsert');
            const link = JSON.stringify({ id: 1, type_id: 1 });
            const result = yield exports.api.sql((0, sql_1.sql) `select links__sync__handler__prepare__function('${link}'::jsonb, ${handlerId}::bigint)`);
            log('prepare result', (_c = (_b = (_a = result === null || result === void 0 ? void 0 : result.data) === null || _a === void 0 ? void 0 : _a.result) === null || _b === void 0 ? void 0 : _b[1]) === null || _c === void 0 ? void 0 : _c[0]);
        }));
    });
    describe('DeepClient mini', () => {
        it(`id`, () => __awaiter(void 0, void 0, void 0, function* () {
            var _a, _b, _c, _d, _e, _f, _g;
            const result = yield exports.api.sql((0, sql_1.sql) `select links__deep__client(${yield deep.id('deep', 'admin')}::bigint, 'id', '["@deep-foundation/core", "Rule"]'::jsonb)`);
            const clientResult = yield deep.id('@deep-foundation/core', 'Rule');
            log('id result', (_c = (_b = (_a = result === null || result === void 0 ? void 0 : result.data) === null || _a === void 0 ? void 0 : _a.result) === null || _b === void 0 ? void 0 : _b[1]) === null || _c === void 0 ? void 0 : _c[0]);
            chai_1.assert.equal((_g = JSON.parse((_f = (_e = (_d = result === null || result === void 0 ? void 0 : result.data) === null || _d === void 0 ? void 0 : _d.result) === null || _e === void 0 ? void 0 : _e[1]) === null || _f === void 0 ? void 0 : _f[0])) === null || _g === void 0 ? void 0 : _g[0], clientResult);
        }));
        describe('select', () => {
            it(`select by id and type_id`, () => __awaiter(void 0, void 0, void 0, function* () {
                var _a, _b, _c, _d, _e, _f;
                const debug = log.extend('selectByIdAndType');
                const anyTypeId = yield deep.id('@deep-foundation/core', 'Type');
                const inserted = yield deep.insert({ type_id: 1, from_id: anyTypeId, to_id: anyTypeId });
                const customLinkId = (_b = (_a = inserted === null || inserted === void 0 ? void 0 : inserted.data) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.id;
                debug('inserted', inserted);
                let result;
                try {
                    result = yield exports.api.sql((0, sql_1.sql) `select links__deep__client(${yield deep.id('deep', 'admin')}::bigint, 'select', '{"id": "${customLinkId}", "type_id":1}'::jsonb)`);
                }
                catch (e) {
                    debug('select error: ', e);
                }
                const selectedByDeep = (_c = (yield deep.select(customLinkId))) === null || _c === void 0 ? void 0 : _c.data.map((_a) => {
                    var { __typename } = _a, filtered = __rest(_a, ["__typename"]);
                    return filtered;
                });
                debug('delete inserted', yield deep.delete({ id: { _eq: customLinkId } }));
                const selectedBySql = JSON.parse((_f = (_e = (_d = result === null || result === void 0 ? void 0 : result.data) === null || _d === void 0 ? void 0 : _d.result) === null || _e === void 0 ? void 0 : _e[1]) === null || _f === void 0 ? void 0 : _f[0]).map((link) => { return { id: Number(link.id), value: link.value, to_id: Number(link.to_id), from_id: Number(link.from_id), type_id: Number(link.type_id) }; });
                debug('selectedBySql', selectedBySql);
                chai_1.assert.equal(selectedBySql.length, 1);
                debug('selectedBySql?.[0]', selectedBySql === null || selectedBySql === void 0 ? void 0 : selectedBySql[0]);
                chai_1.assert.deepEqual(selectedBySql, selectedByDeep);
            }));
            it.skip(`select by only value`, () => __awaiter(void 0, void 0, void 0, function* () {
                var _g, _h, _j, _k, _l, _m;
                const debug = log.extend('selectByValue');
                const value = 'testValue';
                const anyTypeId = yield deep.id('@deep-foundation/core', 'Type');
                const inserted = yield deep.insert({ type_id: 1, from_id: anyTypeId, to_id: anyTypeId });
                const customLinkId = (_h = (_g = inserted === null || inserted === void 0 ? void 0 : inserted.data) === null || _g === void 0 ? void 0 : _g[0]) === null || _h === void 0 ? void 0 : _h.id;
                debug('inserted', inserted);
                let result;
                try {
                    result = yield exports.api.sql((0, sql_1.sql) `select links__deep__client(${yield deep.id('deep', 'admin')}::bigint, 'select', '{"value":"${value}"}'::jsonb)`);
                }
                catch (e) {
                    debug('select error: ', e);
                }
                const selectedByDeep = (_j = (yield deep.select({ value: { _eq: value } }))) === null || _j === void 0 ? void 0 : _j.data.map((_a) => {
                    var { __typename } = _a, filtered = __rest(_a, ["__typename"]);
                    return filtered;
                });
                debug('delete inserted', yield deep.delete({ id: { _eq: customLinkId } }));
                const selectedBySql = JSON.parse((_m = (_l = (_k = result === null || result === void 0 ? void 0 : result.data) === null || _k === void 0 ? void 0 : _k.result) === null || _l === void 0 ? void 0 : _l[1]) === null || _m === void 0 ? void 0 : _m[0]).map((link) => { return { id: Number(link.id), value: link.value, to_id: Number(link.to_id), from_id: Number(link.from_id), type_id: Number(link.type_id) }; });
                debug('selectedBySql', selectedBySql);
                chai_1.assert.equal(selectedBySql.length, 1);
                debug('selectedBySql?.[0]', selectedBySql === null || selectedBySql === void 0 ? void 0 : selectedBySql[0]);
                chai_1.assert.deepEqual(selectedBySql, selectedByDeep);
            }));
        });
        it(`insert`, () => __awaiter(void 0, void 0, void 0, function* () {
            var _h, _j, _k, _l, _m, _o, _p, _q, _r, _s;
            const result = yield exports.api.sql((0, sql_1.sql) `select links__deep__client(${yield deep.id('deep', 'admin')}::bigint, 'insert', '{"type_id":1}'::jsonb)`);
            const customLinkId = (_o = (_m = (_l = JSON.parse((_k = (_j = (_h = result === null || result === void 0 ? void 0 : result.data) === null || _h === void 0 ? void 0 : _h.result) === null || _j === void 0 ? void 0 : _j[1]) === null || _k === void 0 ? void 0 : _k[0])) === null || _l === void 0 ? void 0 : _l.data) === null || _m === void 0 ? void 0 : _m[0]) === null || _o === void 0 ? void 0 : _o.id;
            log('customLinkId', customLinkId);
            const clientResult = yield deep.select({ id: { _eq: customLinkId } });
            log('clientResult', clientResult);
            if (customLinkId === ((_q = (_p = clientResult === null || clientResult === void 0 ? void 0 : clientResult.data) === null || _p === void 0 ? void 0 : _p[0]) === null || _q === void 0 ? void 0 : _q.id))
                deep.delete({ id: { _eq: customLinkId } });
            chai_1.assert.equal(customLinkId, (_s = (_r = clientResult === null || clientResult === void 0 ? void 0 : clientResult.data) === null || _r === void 0 ? void 0 : _r[0]) === null || _s === void 0 ? void 0 : _s.id);
        }));
        it.skip(`update`, () => __awaiter(void 0, void 0, void 0, function* () {
            var _t, _u, _v, _w, _x, _y;
            const inserted = yield deep.insert({ type_id: 1 });
            const customLinkId = (_u = (_t = inserted === null || inserted === void 0 ? void 0 : inserted.data) === null || _t === void 0 ? void 0 : _t[0]) === null || _u === void 0 ? void 0 : _u.id;
            log('inserted', inserted);
            const result = yield exports.api.sql((0, sql_1.sql) `select links__deep__client(${yield deep.id('deep', 'admin')}::bigint, 'update', '{"id": "${customLinkId}", "_set": { "to_id":${customLinkId}, "from_id": ${customLinkId} } }'::jsonb)`);
            log('result', (_x = (_w = (_v = result === null || result === void 0 ? void 0 : result.data) === null || _v === void 0 ? void 0 : _v.result) === null || _w === void 0 ? void 0 : _w[1]) === null || _x === void 0 ? void 0 : _x[0]);
            const clientResult = yield deep.select({ id: { _eq: customLinkId } });
            chai_1.assert.equal((_y = clientResult === null || clientResult === void 0 ? void 0 : clientResult.data[0]) === null || _y === void 0 ? void 0 : _y.type_id, 2);
        }));
        it(`delete`, () => __awaiter(void 0, void 0, void 0, function* () {
            var _z, _0;
            const inserted = yield deep.insert({ type_id: 2 });
            const customLinkId = (_0 = (_z = inserted === null || inserted === void 0 ? void 0 : inserted.data) === null || _z === void 0 ? void 0 : _z[0]) === null || _0 === void 0 ? void 0 : _0.id;
            yield exports.api.sql((0, sql_1.sql) `select links__deep__client(${yield deep.id('deep', 'admin')}::bigint, 'delete', '{"id": "${customLinkId}"}'::jsonb)`);
            const result = yield deep.select({ id: { _eq: customLinkId } });
            log('delete result', result);
            chai_1.assert.equal(result === null || result === void 0 ? void 0 : result.data[0], undefined);
        }));
    });
    describe('Handle operations', () => {
        describe('Handle insert', () => {
            it(`Handle insert on type`, () => __awaiter(void 0, void 0, void 0, function* () {
                var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m;
                const debug = log.extend('HandleInsert');
                const typeId = yield deep.id('@deep-foundation/core', 'Type');
                const handleInsertTypeId = yield deep.id('@deep-foundation/core', 'HandleInsert');
                const supportsId = yield deep.id('@deep-foundation/core', 'plv8SupportsJs');
                const anyTypeId = yield deep.id('@deep-foundation/core', 'Type');
                const inserted = yield deep.insert({ type_id: 1, from_id: anyTypeId, to_id: anyTypeId });
                const customLinkId = (_b = (_a = inserted === null || inserted === void 0 ? void 0 : inserted.data) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.id;
                const handler = yield (0, handlers_1.insertHandler)(handleInsertTypeId, typeId, `({deep, data}) => { deep.insert({type_id: ${customLinkId}, to_id: ${customLinkId}, from_id: ${customLinkId}}); }`, undefined, supportsId);
                debug('handler', handler);
                debug('customLinkId', customLinkId);
                try {
                    const linkId = (_d = (_c = (yield deep.insert({ type_id: typeId }))) === null || _c === void 0 ? void 0 : _c.data) === null || _d === void 0 ? void 0 : _d[0].id;
                    debug('linkId', linkId);
                    debug('delete linkid', yield deep.delete({ id: { _eq: linkId } }));
                }
                catch (e) {
                    debug('insert error: ', e);
                }
                const insertedByHandler = yield deep.select({ type_id: { _eq: customLinkId }, to_id: { _eq: customLinkId }, from_id: { _eq: customLinkId } });
                debug('insertedByHandler', (_f = (_e = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _e === void 0 ? void 0 : _e[0]) === null || _f === void 0 ? void 0 : _f.id);
                if ((_h = (_g = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _g === void 0 ? void 0 : _g[0]) === null || _h === void 0 ? void 0 : _h.id)
                    yield deep.delete((_k = (_j = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _j === void 0 ? void 0 : _j[0]) === null || _k === void 0 ? void 0 : _k.id);
                yield deep.delete(customLinkId);
                debug('delete handler', yield (0, handlers_1.deleteHandler)(handler));
                chai_1.assert.equal(!!((_m = (_l = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _l === void 0 ? void 0 : _l[0]) === null || _m === void 0 ? void 0 : _m.id), true);
            }));
            it(`Handle insert 2 triggers and broke transaction in second`, () => __awaiter(void 0, void 0, void 0, function* () {
                var _o, _p, _q, _r, _s, _t, _u, _v, _w, _x, _y, _z;
                const debug = log.extend('HandleInsert');
                const typeId = yield deep.id('@deep-foundation/core', 'Type');
                const handleInsertTypeId = yield deep.id('@deep-foundation/core', 'HandleInsert');
                const supportsId = yield deep.id('@deep-foundation/core', 'plv8SupportsJs');
                const anyTypeId = yield deep.id('@deep-foundation/core', 'Type');
                const inserted = yield deep.insert({ type_id: 1, from_id: anyTypeId, to_id: anyTypeId });
                const customLinkId = (_p = (_o = inserted === null || inserted === void 0 ? void 0 : inserted.data) === null || _o === void 0 ? void 0 : _o[0]) === null || _p === void 0 ? void 0 : _p.id;
                const handler = yield (0, handlers_1.insertHandler)(handleInsertTypeId, typeId, `({deep, data}) => { deep.insert({type_id: ${customLinkId}, to_id: ${customLinkId}, from_id: ${customLinkId}}); }`, undefined, supportsId);
                debug('handler', handler);
                debug('customLinkId', customLinkId);
                const handler2 = yield (0, handlers_1.insertHandler)(handleInsertTypeId, typeId, `({deep, data}) => { deep.insert({type_id: ${customLinkId}, to_id: ${customLinkId}, from_id: ${customLinkId}}); throw new Error('errorTest')}`, undefined, supportsId);
                try {
                    const linkId = (_r = (_q = (yield deep.insert({ type_id: typeId }))) === null || _q === void 0 ? void 0 : _q.data) === null || _r === void 0 ? void 0 : _r[0].id;
                    debug('linkId', linkId);
                }
                catch (e) {
                    debug('insert error: ', e);
                }
                const insertedByHandler = yield deep.select({ type_id: { _eq: customLinkId }, to_id: { _eq: customLinkId }, from_id: { _eq: customLinkId } });
                debug('insertedByHandler', (_t = (_s = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _s === void 0 ? void 0 : _s[0]) === null || _t === void 0 ? void 0 : _t.id);
                if ((_v = (_u = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _u === void 0 ? void 0 : _u[0]) === null || _v === void 0 ? void 0 : _v.id)
                    yield deep.delete((_x = (_w = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _w === void 0 ? void 0 : _w[0]) === null || _x === void 0 ? void 0 : _x.id);
                yield deep.delete(customLinkId);
                debug('delete handler', yield (0, handlers_1.deleteHandler)(handler));
                debug('delete handler2', yield (0, handlers_1.deleteHandler)(handler2));
                chai_1.assert.equal((_z = (_y = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _y === void 0 ? void 0 : _y[0]) === null || _z === void 0 ? void 0 : _z.id, undefined);
            }));
            it(`Handle insert 1 trigger and 2 types, check not triggered twice`, () => __awaiter(void 0, void 0, void 0, function* () {
                var _0, _1, _2, _3, _4, _5, _6, _7, _8, _9, _10, _11, _12, _13;
                const debug = log.extend('HandleInsert1x2');
                const handleInsertTypeId = yield deep.id('@deep-foundation/core', 'HandleInsert');
                const supportsId = yield deep.id('@deep-foundation/core', 'plv8SupportsJs');
                const anyTypeId = yield deep.id('@deep-foundation/core', 'Type');
                const inserted = yield deep.insert({ type_id: 1, from_id: anyTypeId, to_id: anyTypeId });
                const customLinkId = (_1 = (_0 = inserted === null || inserted === void 0 ? void 0 : inserted.data) === null || _0 === void 0 ? void 0 : _0[0]) === null || _1 === void 0 ? void 0 : _1.id;
                debug('customLinkId', customLinkId);
                const inserted2 = yield deep.insert({ type_id: 1 });
                const customLinkId2 = (_3 = (_2 = inserted2 === null || inserted2 === void 0 ? void 0 : inserted2.data) === null || _2 === void 0 ? void 0 : _2[0]) === null || _3 === void 0 ? void 0 : _3.id;
                debug('customLinkId2', customLinkId2);
                const handler = yield (0, handlers_1.insertHandler)(handleInsertTypeId, customLinkId2, `({deep, data}) => { deep.insert({type_id: ${customLinkId}, to_id: ${customLinkId}, from_id: ${customLinkId}}); }`, undefined, supportsId);
                debug('handler', handler);
                try {
                    const link = (_5 = (_4 = (yield deep.insert({ type_id: customLinkId2 }))) === null || _4 === void 0 ? void 0 : _4.data) === null || _5 === void 0 ? void 0 : _5[0];
                    const linkId = link.id;
                    if (linkId)
                        yield deep.delete(linkId);
                    debug('linkId', linkId);
                }
                catch (e) {
                    debug('insert error: ', e);
                }
                const insertedByHandler = yield deep.select({ type_id: { _eq: customLinkId }, to_id: { _eq: customLinkId }, from_id: { _eq: customLinkId } });
                chai_1.assert.equal((_6 = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _6 === void 0 ? void 0 : _6.length, 1);
                const link2 = (_8 = (_7 = (yield deep.insert({ type_id: customLinkId, from_id: customLinkId, to_id: customLinkId }))) === null || _7 === void 0 ? void 0 : _7.data) === null || _8 === void 0 ? void 0 : _8[0];
                const insertedByHandler2 = yield deep.select({ type_id: { _eq: customLinkId }, to_id: { _eq: customLinkId }, from_id: { _eq: customLinkId } });
                chai_1.assert.equal((_9 = insertedByHandler2 === null || insertedByHandler2 === void 0 ? void 0 : insertedByHandler2.data) === null || _9 === void 0 ? void 0 : _9.length, 2);
                if (link2 === null || link2 === void 0 ? void 0 : link2.id)
                    yield deep.delete(link2 === null || link2 === void 0 ? void 0 : link2.id);
                debug('insertedByHandler', insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data);
                if ((_11 = (_10 = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _10 === void 0 ? void 0 : _10[0]) === null || _11 === void 0 ? void 0 : _11.id)
                    yield deep.delete((_13 = (_12 = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _12 === void 0 ? void 0 : _12[0]) === null || _13 === void 0 ? void 0 : _13.id);
                yield deep.delete(customLinkId);
                yield deep.delete(customLinkId2);
                debug('delete handler', yield (0, handlers_1.deleteHandler)(handler));
            }));
            it(`Handle insert on type throw error`, () => __awaiter(void 0, void 0, void 0, function* () {
                var _14, _15, _16, _17, _18, _19, _20, _21, _22, _23;
                const debug = log.extend('HandleInsertError');
                const typeId = yield deep.id('@deep-foundation/core', 'Type');
                const handleInsertTypeId = yield deep.id('@deep-foundation/core', 'HandleInsert');
                const supportsId = yield deep.id('@deep-foundation/core', 'plv8SupportsJs');
                const anyTypeId = yield deep.id('@deep-foundation/core', 'Type');
                const inserted = yield deep.insert({ type_id: 1, from_id: anyTypeId, to_id: anyTypeId });
                const customLinkId = (_15 = (_14 = inserted === null || inserted === void 0 ? void 0 : inserted.data) === null || _14 === void 0 ? void 0 : _14[0]) === null || _15 === void 0 ? void 0 : _15.id;
                const handler = yield (0, handlers_1.insertHandler)(handleInsertTypeId, typeId, `({deep, data}) => { deep.insert({type_id: ${customLinkId}, to_id: ${customLinkId}, from_id: ${customLinkId}}); throw new Error('testError');}`, undefined, supportsId);
                let linkId;
                let error;
                debug('handler', handler);
                try {
                    const linkId = (_17 = (_16 = (yield deep.insert({ type_id: typeId }))) === null || _16 === void 0 ? void 0 : _16.data) === null || _17 === void 0 ? void 0 : _17[0].id;
                    debug('ensureLinkIsCreated', linkId);
                    throw new Error('Not errored hadnler!');
                }
                catch (e) {
                    debug('error', e === null || e === void 0 ? void 0 : e.message);
                    error = e === null || e === void 0 ? void 0 : e.message;
                }
                const insertedByHandler = yield deep.select({ type_id: { _eq: customLinkId }, to_id: { _eq: customLinkId }, from_id: { _eq: customLinkId } });
                debug('insertedByHandler', (_19 = (_18 = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _18 === void 0 ? void 0 : _18[0]) === null || _19 === void 0 ? void 0 : _19.id);
                debug('delete handler', JSON.stringify(yield (0, handlers_1.deleteHandler)(handler)));
                if ((_21 = (_20 = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _20 === void 0 ? void 0 : _20[0]) === null || _21 === void 0 ? void 0 : _21.id)
                    yield deep.delete((_23 = (_22 = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _22 === void 0 ? void 0 : _22[0]) === null || _23 === void 0 ? void 0 : _23.id);
                yield deep.delete(customLinkId);
                if (linkId) {
                    const deleteResult = yield deep.delete({ id: { _eq: linkId } });
                    debug('delete linkid', deleteResult);
                }
                chai_1.assert.equal(error, 'testError');
            }));
            it(`Handle insert on selector`, () => __awaiter(void 0, void 0, void 0, function* () {
                var _24, _25, _26, _27, _28, _29, _30, _31, _32, _33;
                const debug = log.extend('HandleInsertSelect');
                const handleInsertTypeId = yield deep.id('@deep-foundation/core', 'HandleInsert');
                const supportsId = yield deep.id('@deep-foundation/core', 'plv8SupportsJs');
                const selector = yield (0, handlers_1.insertSelector)();
                debug('selector', selector);
                const { nodeTypeId, linkTypeId, treeId, selectorId, rootId } = selector;
                const anyTypeId = yield deep.id('@deep-foundation/core', 'Type');
                const inserted = yield deep.insert({ type_id: 1, from_id: anyTypeId, to_id: anyTypeId });
                const customLinkId = (_25 = (_24 = inserted === null || inserted === void 0 ? void 0 : inserted.data) === null || _24 === void 0 ? void 0 : _24[0]) === null || _25 === void 0 ? void 0 : _25.id;
                const handler = yield (0, handlers_1.insertHandler)(handleInsertTypeId, selectorId, `({deep, data}) => { deep.insert({type_id: ${customLinkId}, to_id: ${customLinkId}, from_id: ${customLinkId}}); }`, undefined, supportsId);
                let selectorItem;
                debug('handler', handler);
                try {
                    selectorItem = yield (0, handlers_1.insertSelectorItem)({ selectorId, nodeTypeId, linkTypeId, treeId, rootId });
                    debug('selectorItem', selectorItem);
                }
                catch (e) {
                    error(e);
                }
                if (selectorItem === null || selectorItem === void 0 ? void 0 : selectorItem.linkId)
                    yield deep.delete(selectorItem.linkId);
                const insertedByHandler = yield deep.select({ type_id: { _eq: customLinkId }, to_id: { _eq: customLinkId }, from_id: { _eq: customLinkId } });
                debug('insertedByHandler', (_27 = (_26 = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _26 === void 0 ? void 0 : _26[0]) === null || _27 === void 0 ? void 0 : _27.id);
                if ((_29 = (_28 = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _28 === void 0 ? void 0 : _28[0]) === null || _29 === void 0 ? void 0 : _29.id)
                    yield deep.delete((_31 = (_30 = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _30 === void 0 ? void 0 : _30[0]) === null || _31 === void 0 ? void 0 : _31.id);
                yield deep.delete(customLinkId);
                yield deep.delete(customLinkId);
                debug('deleteSelector');
                yield (0, handlers_1.deleteSelector)(selector);
                debug('deleteHandler');
                yield (0, handlers_1.deleteHandler)(handler);
                chai_1.assert.equal(!!((_33 = (_32 = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _32 === void 0 ? void 0 : _32[0]) === null || _33 === void 0 ? void 0 : _33.id), true);
            }));
        });
        describe.skip('Handle update', () => {
            it(`Handle update on type`, () => __awaiter(void 0, void 0, void 0, function* () {
                var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
                const debug = log.extend('HandleUpdate');
                const typeId = yield deep.id('@deep-foundation/core', 'Type');
                const HandlerTypeId = yield deep.id('@deep-foundation/core', 'Handler');
                const handleUpdateTypeId = yield deep.id('@deep-foundation/core', 'HandleUpdate');
                const supportsId = yield deep.id('@deep-foundation/core', 'plv8SupportsJs');
                const anyTypeId = yield deep.id('@deep-foundation/core', 'Type');
                const inserted = yield deep.insert({ type_id: 1, from_id: anyTypeId, to_id: anyTypeId });
                const customLinkId = (_b = (_a = inserted === null || inserted === void 0 ? void 0 : inserted.data) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.id;
                const handler = yield (0, handlers_1.insertHandler)(handleUpdateTypeId, typeId, `({deep, data}) => { deep.insert({type_id: ${customLinkId}, to_id: ${customLinkId}, from_id: ${customLinkId}}); }`, undefined, supportsId);
                debug('handler', handler);
                debug('customLinkId', customLinkId);
                const linkId = (_d = (_c = (yield deep.insert({ type_id: typeId }))) === null || _c === void 0 ? void 0 : _c.data) === null || _d === void 0 ? void 0 : _d[0].id;
                try {
                    const updated = yield deep.update(linkId, { type_id: HandlerTypeId });
                    debug('updated', updated);
                }
                catch (e) {
                    debug('insert error: ', e);
                }
                const insertedByHandler = yield deep.select({ type_id: { _eq: customLinkId }, to_id: { _eq: customLinkId }, from_id: { _eq: customLinkId } });
                debug('insertedByHandler', insertedByHandler);
                debug('delete linkid', yield deep.delete({ id: { _eq: linkId } }));
                if ((_f = (_e = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _e === void 0 ? void 0 : _e[0]) === null || _f === void 0 ? void 0 : _f.id)
                    yield deep.delete((_h = (_g = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _g === void 0 ? void 0 : _g[0]) === null || _h === void 0 ? void 0 : _h.id);
                yield deep.delete(customLinkId);
                debug('insertedByHandler', insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data[0]);
                debug('delete handler', yield (0, handlers_1.deleteHandler)(handler));
                chai_1.assert.equal(!!((_k = (_j = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _j === void 0 ? void 0 : _j[0]) === null || _k === void 0 ? void 0 : _k.id), true);
            }));
            it(`Handle update on selector`, () => __awaiter(void 0, void 0, void 0, function* () {
                var _l, _m, _o, _p, _q, _r, _s, _t;
                const debug = log.extend('HandleUpdateSelect');
                const HandlerTypeId = yield deep.id('@deep-foundation/core', 'Handler');
                const handleUpdateTypeId = yield deep.id('@deep-foundation/core', 'HandleUpdate');
                const supportsId = yield deep.id('@deep-foundation/core', 'plv8SupportsJs');
                const selector = yield (0, handlers_1.insertSelector)();
                debug('selector', selector);
                const { nodeTypeId, linkTypeId, treeId, selectorId, rootId } = selector;
                const anyTypeId = yield deep.id('@deep-foundation/core', 'Type');
                const inserted = yield deep.insert({ type_id: 1, from_id: anyTypeId, to_id: anyTypeId });
                const customLinkId = (_m = (_l = inserted === null || inserted === void 0 ? void 0 : inserted.data) === null || _l === void 0 ? void 0 : _l[0]) === null || _m === void 0 ? void 0 : _m.id;
                const handler = yield (0, handlers_1.insertHandler)(handleUpdateTypeId, selectorId, `({deep, data}) => { deep.insert({type_id: ${customLinkId}, to_id: ${customLinkId}, from_id: ${customLinkId}}); }`, undefined, supportsId);
                const selectorItem = yield (0, handlers_1.insertSelectorItem)({ selectorId, nodeTypeId, linkTypeId, treeId, rootId });
                debug('handler', handler);
                try {
                    const updated = yield deep.update(selectorItem.linkId, { type_id: HandlerTypeId });
                    debug('updated', updated);
                }
                catch (e) {
                    error(e);
                }
                const insertedByHandler = yield deep.select({ type_id: { _eq: customLinkId }, to_id: { _eq: customLinkId }, from_id: { _eq: customLinkId } });
                debug('insertedByHandler', insertedByHandler);
                if ((_p = (_o = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _o === void 0 ? void 0 : _o[0]) === null || _p === void 0 ? void 0 : _p.id)
                    yield deep.delete((_r = (_q = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _q === void 0 ? void 0 : _q[0]) === null || _r === void 0 ? void 0 : _r.id);
                yield deep.delete(customLinkId);
                debug('selectorItem');
                if (selectorItem === null || selectorItem === void 0 ? void 0 : selectorItem.linkId)
                    yield deep.delete(selectorItem.linkId);
                debug('deleteSelector');
                yield (0, handlers_1.deleteSelector)(selector);
                debug('deleteHandler');
                yield (0, handlers_1.deleteHandler)(handler);
                chai_1.assert.equal(!!((_t = (_s = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _s === void 0 ? void 0 : _s[0]) === null || _t === void 0 ? void 0 : _t.id), true);
            }));
        });
        describe('Handle delete', () => {
            it(`Handle delete on type`, () => __awaiter(void 0, void 0, void 0, function* () {
                var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
                const debug = log.extend('HandleDelete');
                const typeId = yield deep.id('@deep-foundation/core', 'Type');
                const handleDeleteTypeId = yield deep.id('@deep-foundation/core', 'HandleDelete');
                const supportsId = yield deep.id('@deep-foundation/core', 'plv8SupportsJs');
                const anyTypeId = yield deep.id('@deep-foundation/core', 'Type');
                const inserted = yield deep.insert({ type_id: 1, from_id: anyTypeId, to_id: anyTypeId });
                const customLinkId = (_b = (_a = inserted === null || inserted === void 0 ? void 0 : inserted.data) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.id;
                const handler = yield (0, handlers_1.insertHandler)(handleDeleteTypeId, typeId, `({deep, data}) => { deep.insert({type_id: ${customLinkId}, to_id: ${customLinkId}, from_id: ${customLinkId}}); }`, undefined, supportsId);
                debug('handler', handler);
                debug('customLinkId', customLinkId);
                const linkId = (_d = (_c = (yield deep.insert({ type_id: typeId }))) === null || _c === void 0 ? void 0 : _c.data) === null || _d === void 0 ? void 0 : _d[0].id;
                try {
                    const deleted = yield deep.delete(linkId);
                    debug('deleted', deleted);
                }
                catch (e) {
                    debug('insert error: ', e);
                }
                const insertedByHandler = yield deep.select({ type_id: { _eq: customLinkId }, to_id: { _eq: customLinkId }, from_id: { _eq: customLinkId } });
                debug('insertedByHandler', insertedByHandler);
                if ((_f = (_e = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _e === void 0 ? void 0 : _e[0]) === null || _f === void 0 ? void 0 : _f.id)
                    yield deep.delete((_h = (_g = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _g === void 0 ? void 0 : _g[0]) === null || _h === void 0 ? void 0 : _h.id);
                debug('delete handler', yield (0, handlers_1.deleteHandler)(handler));
                yield deep.delete(customLinkId);
                chai_1.assert.equal(!!((_k = (_j = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _j === void 0 ? void 0 : _j[0]) === null || _k === void 0 ? void 0 : _k.id), true);
            }));
            it(`Handle delete on selector`, () => __awaiter(void 0, void 0, void 0, function* () {
                var _l, _m, _o, _p, _q, _r, _s, _t;
                const debug = log.extend('HandleDeleteSelect');
                const handleDeleteTypeId = yield deep.id('@deep-foundation/core', 'HandleDelete');
                const supportsId = yield deep.id('@deep-foundation/core', 'plv8SupportsJs');
                const selector = yield (0, handlers_1.insertSelector)();
                debug('selector', selector);
                const { nodeTypeId, linkTypeId, treeId, selectorId, rootId } = selector;
                const anyTypeId = yield deep.id('@deep-foundation/core', 'Type');
                const inserted = yield deep.insert({ type_id: 1, from_id: anyTypeId, to_id: anyTypeId });
                const customLinkId = (_m = (_l = inserted === null || inserted === void 0 ? void 0 : inserted.data) === null || _l === void 0 ? void 0 : _l[0]) === null || _m === void 0 ? void 0 : _m.id;
                const handler = yield (0, handlers_1.insertHandler)(handleDeleteTypeId, selectorId, `({deep, data}) => { deep.insert({type_id: ${customLinkId}, to_id: ${customLinkId}, from_id: ${customLinkId}}); }`, undefined, supportsId);
                const selectorItem = yield (0, handlers_1.insertSelectorItem)({ selectorId, nodeTypeId, linkTypeId, treeId, rootId });
                debug('handler', handler);
                try {
                    const deleted = yield deep.delete(selectorItem.linkId);
                    debug('deleted', deleted);
                }
                catch (e) {
                    error(e);
                }
                const insertedByHandler = yield deep.select({ type_id: { _eq: customLinkId }, to_id: { _eq: customLinkId }, from_id: { _eq: customLinkId } });
                debug('insertedByHandler', insertedByHandler);
                if ((_p = (_o = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _o === void 0 ? void 0 : _o[0]) === null || _p === void 0 ? void 0 : _p.id)
                    yield deep.delete((_r = (_q = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _q === void 0 ? void 0 : _q[0]) === null || _r === void 0 ? void 0 : _r.id);
                yield deep.delete(customLinkId);
                debug('deleteSelector');
                yield (0, handlers_1.deleteSelector)(selector);
                debug('deleteHandler');
                yield (0, handlers_1.deleteHandler)(handler);
                chai_1.assert.equal(!!((_t = (_s = insertedByHandler === null || insertedByHandler === void 0 ? void 0 : insertedByHandler.data) === null || _s === void 0 ? void 0 : _s[0]) === null || _t === void 0 ? void 0 : _t.id), true);
            }));
        });
    });
});
//# sourceMappingURL=sync-handlers.js.map